# w.c.s. - web application for online forms
# Copyright (C) 2005-2019  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys

from django.core.management.base import BaseCommand, CommandError

from wcs.qommon.publisher import UnknownTenantError, get_publisher_class


class TenantCommand(BaseCommand):
    def init_tenant_publisher(self, domain, **kwargs):
        publisher = get_publisher_class().create_publisher(**kwargs)
        try:
            publisher.set_tenant_by_hostname(domain)
        except UnknownTenantError:
            raise CommandError('unknown tenant')
        publisher.install_lang()
        publisher.substitutions.feed(publisher)
        return publisher

    def disable_sentry(self):
        # disable sentry by reinitialising with no parameters
        if 'sentry_sdk' in sys.modules:
            # noqa pylint: disable=import-error
            import sentry_sdk

            sentry_sdk.init()
